import json

from seclea_ai.lib.seclea_utils.core.transmission import Transmission

# from ..errors import throws_api_err

root = "/collection/datasets"


# @throws_api_err
def post_dataset(
    transmission: Transmission,
    dataset_file_path: str,
    project_pk: str,
    organization_pk: str,
    name: str,
    metadata: dict,
    dataset_hash: str,
    parent_dataset_hash: str = None,
    delete=False,
):
    """
    @param transmission:
    @param dataset_file_path:
    @param project_pk:
    @param organization_pk:
    @param name:
    @param metadata:
    @param dataset_hash:
    @param parent_dataset_hash:
    @param delete: delete dataset file
    @return:
    """
    dataset_queryparams = {
        "project": project_pk,
        "organization": organization_pk,
        "name": name,
        "metadata": json.dumps(metadata),
        "hash": str(dataset_hash),
        "parent": parent_dataset_hash,
    }
    res = transmission.send_file(
        url_path=f"{root}",
        file_path=dataset_file_path,
        query_params=dataset_queryparams,
        delete_file=delete,
    )
    return res



# import os
# import unittest
#
# from unittest import mock
# from unittest.mock import mock_open, patch
#
# import responses
#
# from seclea_ai import SecleaAI
#
#
# base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
# folder_path = os.path.join(base_dir, "")
# print(folder_path)
#
#
# class TestSecleaAI(unittest.TestCase):
#     @responses.activate
#     @mock.patch("seclea_ai.authentication.getpass", return_value="test_pass")
#     @mock.patch("builtins.input", autospec=True, return_value="test_user")
#     def test_init_seclea_object(self, mock_input, mock_getpass) -> None:
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/refresh/",
#             json={"access": "dummy_access_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/obtain/",
#             json={"access": "dummy_access_token", "refresh": "dummy_refresh_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/projects?name=test-project",
#             json=[{"id": 1, "name": "test-project"}],
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/models",
#             json=[{"id": 1, "name": "GBM-1"}],
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/datasets",
#             json=[{"id": 1, "name": "Fraud detection"}],
#             status=200,
#         )
#         with patch(
#             "builtins.open",
#             new=mock_open(
#                 read_data='{"refresh": "dummy_refresh_token", "username": "test_user"}\n'
#             ),
#         ) as mock_file:
#             SecleaAI(
#                 project_name="test-project",
#                 organization="Onespan",
#                 platform_url="http://localhost:8000",
#                 auth_url="http://localhost:8010",
#             )
#         mock_file.assert_called()
#
#     @responses.activate
#     @mock.patch("seclea_ai.authentication.getpass", return_value="test_pass")
#     @mock.patch("builtins.input", autospec=True, return_value="test_user")
#     def test_getting_project_fail(self, mock_input, mock_getpass) -> None:
#         """Test using an Project name that does not exist and sending new project fails raises a ValueError"""
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/refresh/",
#             json={"access": "dummy_access_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/obtain/",
#             json={"access": "dummy_access_token", "refresh": "dummy_refresh_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/projects?name=New Project",
#             status=400,
#         )
#         with self.assertRaises(ValueError):
#             SecleaAI(
#                 project_name="New Project",
#                 organization="Onespan",
#                 platform_url="http://localhost:8000",
#                 auth_url="http://localhost:8010",
#             )
#
#     @responses.activate
#     @mock.patch("seclea_ai.authentication.getpass", return_value="test_pass")
#     @mock.patch("builtins.input", autospec=True, return_value="test_user")
#     def test_upload_project_fail(self, mock_input, mock_getpass) -> None:
#         """Test using an Project name that does not exist and sending new project fails raises a ValueError"""
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/refresh/",
#             json={"access": "dummy_access_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8010/api/token/obtain/",
#             json={"access": "dummy_access_token", "refresh": "dummy_refresh_token"},
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/projects?name=New Project",
#             status=200,
#             json=[],
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/models",
#             json=[{"id": 1, "name": "GBM-1"}],
#             status=200,
#         )
#         responses.add(
#             method=responses.GET,
#             url="http://localhost:8000/collection/datasets",
#             json=[{"id": 1, "name": "Fraud detection"}],
#             status=200,
#         )
#         responses.add(
#             method=responses.POST,
#             url="http://localhost:8000/collection/projects",
#             status=403,
#         )
#         with self.assertRaises(ValueError):
#             SecleaAI(
#                 project_name="New Project",
#                 organization="Onespan",
#                 platform_url="http://localhost:8000",
#                 auth_url="http://localhost:8010",
#             )
#
#
# if __name__ == "__main__":
#     unittest.main()



from seclea_ai.lib.seclea_utils.core.transmission import Transmission

root = "/collection/model-states"


def post_model_state(
    transmission: Transmission,
    model_state_file_path: str,
    organization_pk: str,
    project_pk: str,
    training_run_pk: str,
    sequence_num: int,
    final_state,
    delete=False,
):
    """

    @param transmission:
    @param model_state_file_path:
    @param organization_pk:
    @param project_pk:
    @param training_run_pk:
    @param sequence_num:
    @param final_state:
    @param delete:
    @return:
    """
    res = transmission.send_file(
        url_path="/collection/model-states",
        file_path=model_state_file_path,
        query_params={
            "organization": organization_pk,
            "project": project_pk,
            "sequence_num": sequence_num,
            "training_run": training_run_pk,
            "final_state": final_state,
        },
        delete_file=delete,
    )
    return res



from io import StringIO

import pandas as pd

from ..core import load_object


def save_dataset(dataset, path):
    dataset.to_csv(path)


def load_dataset(dataset_path: str, index_col=None):
    """
     # TODO: Extend to different types of datasets
    :param model_dataset:
    :return:
    """

    dataset_str = load_object(dataset_path, as_bytes=False)
    dataset_stream = StringIO(dataset_str)
    return pd.read_csv(dataset_stream, index_col=index_col)


def dataset_hash(dataset, project) -> str:
    return str(hash(pd.util.hash_pandas_object(dataset) + project))



# Original Credit https://stackoverflow.com/questions/23212435/permission-denied-to-write-to-my-temporary-file/63173312#63173312
# Extended by Roger Milroy

import os
import tempfile


class CustomNamedTemporaryFile:
    """
    This custom implementation is needed because of the following limitation of tempfile.NamedTemporaryFile:

    > Whether the name can be used to open the file a second time, while the named temporary file is still open,
    > varies across platforms (it can be so used on Unix; it cannot on Windows NT or later).
    """

    def __init__(self, mode="w+b", delete=True):
        self._mode = mode
        self._delete = delete
        self.name = None

    def __enter__(self):
        # Generate a random temporary file name
        self.name = os.path.join(tempfile.gettempdir(), os.urandom(24).hex())
        # Ensure the file is created
        open(self.name, "x").close()
        # Open the file in the given mode
        self._tempFile = open(self.name, self._mode)
        return self._tempFile

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._tempFile.close()
        if self._delete:
            os.remove(self.name)
